from .utils import PAYLOAD_ABOUT
import requests, time

SCRYFALL_API = 'https://api.scryfall.com/'
EXACT_NAME_API = '{}cards/named'.format(SCRYFALL_API)
BASIC_LAND_TYPES = ['swamp', 'forest', 'island', 'plains', 'mountain']

def is_basic_land(name):
    return name.lower() in BASIC_LAND_TYPES

def get_card_by_name(name):
    r = requests.get(EXACT_NAME_API, params={**PAYLOAD_ABOUT, 'exact': name})
    return r.json()

def get_card_tcgplayer_id(card_data):
    return card_data['tcgplayer_id']

def get_card_prints(card_data):
    set_names = set({})
    url = card_data['prints_search_uri']
    has_more = True

    while has_more:
        r = requests.get(url, params=PAYLOAD_ABOUT)
        res = r.json()
        has_more = res['has_more']

        for card in res['data']:
            if 'paper' in card['games']:
                set_names.add(card['set_name'])

        if has_more:
            url = res['next_page']
            time.sleep(0.05)

    return list(set_names)



import time
from .utils import request_soup

CARDKINDOM_BASE = 'https://www.cardkingdom.com/'
CARDKINGDOM_CARD_API = CARDKINDOM_BASE + 'mtg/{edition}/{card}'

CARDKINDOM_CARD_QUALITIES = ['NM', 'EX', 'VG', 'G']

def CARDKINGDOM_URI(card, edition):
    _card = card.lower()
        .replace(' ', '-')
        .replace(':', '')
        .replace("'",'')
        .replace(',','')
    _edition = edition.lower()
        .replace(' ', '-')
        .replace(':', '')
        .replace("'",'')
        .replace(',','')
    return CARDKINGDOM_CARD_API.format(edition=_edition, card=_card)

def prices(soup):
    card_prices = []
    price_spans = soup.find_all('span', class_="stylePrice")
    for price_span in price_spans:
        price = price_span.text.strip().rstrip('\n').replace('$', '')
        card_prices.append(float(price))
    return card_prices

def availabilities(soup):
    card_availabilities = []
    card_qualities_list = soup.find_all('li', class_='itemAddToCart')
    for card_quality in card_qualities_list:
        if 'outOfStock' in card_quality['class']:
            card_availabilities.append(False)
        else:
            card_availabilities.append(True)
    return card_availabilities

def prices_and_availbility(soup):
    _soup = soup.find('ul', class_='addToCartByType')
    card_prices = prices(_soup)
    card_availabilities = availabilities(_soup)
    zipped = list(zip(CARDKINDOM_CARD_QUALITIES, card_prices, card_availabilities))
    results = {}
    for qual, price, avail in zipped:
        results[qual] = {'availability':avail, 'price':float(price)}
    return results




def handle_edition(edition):
    if edition == 'Kaladesh Inventions':
        return 'masterpiece-series-inventions'
    if edition == 'Unlimited Edition':
        return 'unlimited'
    if 'Judge' in edition:
        return 'promotional'
    if 'Volume' in edition:
        return edition.replace('Volume', 'Vol')
    if 'Commander 2011' == edition:
        return 'commander'
    if 'Limited Edition' in edition:
        return edition.replace('Limited Edition', '').strip()
    if 'Revised Edition' == edition:
        return '3rd-edition'
    return edition

def handle_name(name, edition):
    if edition == 'masterpiece-series-inventions':
        return name + '-KLD'
    if 'promotional' == edition:
        return name + '-judge-foil'
    return name


def price_data_for_card(name, prints):
    prices = {}
    for i, edition in enumerate(prints):
        _edition = handle_edition(edition)
        _name = handle_name(name, _edition)
        url = CARDKINGDOM_URI(_name, _edition)
        try:
            soup = request_soup(url)
            prices[edition] = prices_and_availbility(soup)
            if i != len(prints) - 1:
                time.sleep(0.05)
        except AttributeError:
            pass
            # print(_name, _edition)
    return prices


def cheapest_price(prices, must_be_available=False):
    lowest = float('inf')
    edition = None
    quality = None

    for card_edition, qualities in prices.items():
        for card_quality, price_avail in qualities.items():
            price = price_avail['price']
            avail = price_avail['availability']
            if must_be_available and not avail:
                continue
            if price < lowest:
                lowest = price
                edition = card_edition
                quality = card_quality
    return edition, quality, lowest



import urllib.parse, requests, copy
from bs4 import BeautifulSoup

from selenium import webdriver
from selenium.webdriver.firefox.options import Options


PAYLOAD_ABOUT = {
    'project': 'apaupercary',
    'purpose': 'mtg-community',
    'contact': 'sumner.magruder@zmnh.uni-hamburg.de',
    'good-citizen': '50ms-between-calls'
}

def param_str(url, params={}):
    q = ('?' if '?' not in url else '')
    p = copy.deepcopy(PAYLOAD_ABOUT)
    p.update(params)
    s = ','.join([ '{}={}'.format(k,v) for (k, v) in p.items()])
    return '{}{}'.format(q, s)

def request_soup_js(url, delay=30, params={}):
    '''
    Notes:
        1. Assumes Firefox is install in default location on your system

    Arguments:
        url (str): the page to request
        delay (float): how long to wait to allow JS to activate
        params (dict): optional parameters

    Returns:
        soup (bs4.BeautifulSoup): the soup of the requested url
    '''


    p_st = param_str(url, params)

    options = Options()
    options.headless = True

    driver = webdriver.Firefox(options=options)
    driver.implicitly_wait(delay)
    driver.get(url+p_st)
    soup = BeautifulSoup(driver.page_source, 'lxml')
    driver.quit()
    return soup


def request_soup(url, params={}):
    '''
    Requests the url and returns the BeautifulSoup of the response's text.
    In addition, provides the parameters PAYLOAD_ABOUT to the get request.
    This allows the host to know who is crawling their website, why, and how
    to contact should there be an issue.

    Arguments:
        url (str): the URL to request via the python requests library get method
        params (dict): optional params to pass to the url. These will be in
            addition to PAYLOAD_ABOUT.

    Returns:
        soup (bs4.BeautifulSoup)
    '''
    request = requests.get(url, params={**PAYLOAD_ABOUT, **params})
    soup = BeautifulSoup(request.text, features="lxml")
    return soup



import setuptools

from apaupercary import __name__, __version__

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name=__name__,
    version=__version__,
    author="Sumner Magruder",
    author_email="sumner.magruder@zmnh.uni-hamburg.de",
    description="pauper prices for your budget brews",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://pypi.org/project/{}/".format(__name__),
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3.5",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)
